/* $Id: scrollbar.gen_h,v 1.38 2001/07/15 13:33:38 murrayc Exp $ */

/* scrollbar.h
 *
 * Copyright (C) 1998-1999 The Gtk-- Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk--/range.h>
#include <gtk/gtkscrollbar.h>
#include <gtk/gtkhscrollbar.h>
#include <gtk/gtkvscrollbar.h>


namespace Gtk {

//: Base class for {Gtk::HScrollbar::} and {Gtk::VScrollbar::}.
//- The {\class Gtk::Scrollbar} widget is an abstract base class for {Gtk::HScrollbar::}
//- and {Gtk::VScrollbar::}. It is not very useful in itself.
class Scrollbar : public Range {


public:
  typedef Scrollbar          CppObjectType;
  typedef GtkScrollbar            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkScrollbar* gtkobj();
  const GtkScrollbar* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~Scrollbar();

private:

public:

protected:
  Scrollbar();




protected:
  // impl functions

};

//: A vertical scrollbar.
//- The {\class Gtk::VScrollbar} widget is a widget arranged vertically creating a scroll bar.
//- See {Gtk::Scrollbar::} for details on scrollbars. {Gtk::Adjustment::}
//- pointers may be added to handle the adjustment of the scrollbars or it may be
//- left NULL in which case one will be created you. See {Gtk::Adjustment::} for details.
class VScrollbar : public Scrollbar {


public:
  typedef VScrollbar          CppObjectType;
  typedef GtkVScrollbar            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkVScrollbar* gtkobj();
  const GtkVScrollbar* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~VScrollbar();

private:

public:
  explicit VScrollbar( Adjustment &gtkadjustment);
  VScrollbar();





protected:
  // impl functions

};


//: A horizontal scrollbar.
//- The {\class Gtk::HScrollbar} widget is a widget arranged horizontally creating a scroll bar.
//- See {Gtk::Scrollbar::} for details on scrollbars. {Gtk::Adjustment::}
//- pointers may be added to handle the adjustment of the scrollbars or it may be
//- left NULL in which case one will be created you. See {Gtk::Adjustment::} for details.
class HScrollbar : public Scrollbar {


public:
  typedef HScrollbar          CppObjectType;
  typedef GtkHScrollbar            BaseObjectType;

public:
  //: Returns the underlaying gtk+ object.
  GtkHScrollbar* gtkobj();
  const GtkHScrollbar* gtkobj() const;

  //: Returns true if object is this type.
  static bool isA(Gtk::Object *object);

  virtual ~HScrollbar();

private:

public:
  explicit HScrollbar( Adjustment &gtkadjustment);
  HScrollbar();





protected:
  // impl functions

};



}
